''Module simplex.spin: Serial output driver.
''
''Copyright (C) 2006 Philip C. Pilgrim
'' 
''This program is free software; you can redistribute it and/or modify
''it under the terms of the GNU General Public License as published by
''the Free Software Foundation; either version 2 of the License, or
''(at your option) any later version.
''
''This program is distributed in the hope that it will be useful,
''but WITHOUT ANY WARRANTY; without even the implied warranty of
''MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''GNU General Public License for more details.
''
''You should have received a copy of the GNU General Public License
''along with this program; if not, write to the Free Software
''Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
''
''CONTACT
''
''  phil@phipi.com (First contact may require addition to my whitelist.)
''
''VERSION HISTORY
''
''  0.01 alpha: released 2006.03.22
''
''SUMMARY
''
''  This serial output driver is meant to be a simple, drop-in replacement for FullDuplex.spin.
''  It is OUTPUT ONLY and does not require a separate cog to run in. It can also interface
''  directly to SOME RS232 ports (inverted logic) by starting with a negative baudrate argument: 
''
''    simplex.start(0, 0, -9600)
''
''  With an 80MHz clock, 9600 baud is about the fastest this driver will run. 
''

VAR

  long  BaudClock
  long  TxMask
  byte  invert

PUB start(rxpin, txpin, baud)

'' Initializes the baud delay factor, and the output pin direction and level.
'' (The first argument is included for compatibility with FullDuplex.spin but is ignored.)
'' To specify inverted logic for direct connection to an RS232 port, negate the baud argument.
'' (Note: Not all PC RS232 will respond to 3.3-volt logic.)

  invert := baud < 0
  || baud
  BaudClock := clkfreq / baud
  TxMask := 1 << txpin
  outa := outa & !TxMask | !invert & TxMask
  dira |= TxMask
  return true

PUB stop

'' Changes the tx pin back to an input.

  dira &= !TxMask

PUB rxcheck

'' Simplex, so no receiver.

  return -1

PUB rx

'' Simplex, so no reciever.

  return 0

PUB str(str_addr)

'' Send string

  repeat strsize(str_addr)
    tx(byte[str_addr++])

PUB tx(txbyte) | time

'' Send a single character with two stops bits.

  txbyte := (txbyte | $300) << 2
  time := cnt
  repeat 11
    waitcnt(time += BaudClock)
    outa := outa & !TxMask | (((txbyte >>= 1) & 1 > 0) ^ invert) & TxMask