''Module string_demo: Demo program for "strings" and "first_fit" objects.
'' 
''Copyright (C) 2006 Philip C. Pilgrim
'' 
''This program is free software; you can redistribute it and/or modify
''it under the terms of the GNU General Public License as published by
''the Free Software Foundation; either version 2 of the License, or
''(at your option) any later version.
''
''This program is distributed in the hope that it will be useful,
''but WITHOUT ANY WARRANTY; without even the implied warranty of
''MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''GNU General Public License for more details.
''
''You should have received a copy of the GNU General Public License
''along with this program; if not, write to the Free Software
''Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
''
''CONTACT
''
''  phil@phipi.com (First contact may require addition to my whitelist.)
''
''VERSION HISTORY
''
''  0.01 alpha: released 2006.04.05
''
''SUMMARY
''
''  This program demonstrates the dynamic strings module "strings", which
''  uses "first_fit" to do its memory management. Using a set of rules in
''  a context-free grammar, it generates English sentences -- grammatically
''  correct, but usually nonsensical. These sentences are output via "print"
''  to a serial port pin on the Propeller.
''
CON

  WATCH_ALL     = false         'Set to TRUE to watch the sentence being built.
  CONTINUOUS    = true          'Set to FALSE to build only one sentence per run.
  DELAY         = 5             'Number of seconds to delay between sentences.

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000
  HEAP_SIZE     = 2000
  WRAP          = $10000

VAR

  long  Seed, Heap_array[HEAP_SIZE]
  word  Rules, Sentence
  word  rule, lft, rgt, wrd, def, option
  byte  Hp
  

OBJ

  strg :        "strings"
  print:        "print"

PUB start | j, k, m, n

  Hp := strg.start(@heap_array, HEAP_SIZE)
  print.start(31, 30, -9600)
  print.cls
  Seed := cnt
  
  strg.clone(@Rules, @grammar + WRAP)
  strg.split(@Rules, Rules, "/")
  repeat j from 0 to strg.size(Rules) - 1
    rule := @word[Rules][j]
    strg.split(rule, word[rule], string(" ::= "))
    Rule := @word[word[Rule]][1]
    strg.split(rule, word[rule], "|")
  repeat
    strg.clone(@Sentence, string("<sentence>"))
    repeat while (lft := strg.find_next(Sentence, 0, "<")) => 0
      if WATCH_ALL
        print.f1(string("%s\r"), Sentence)
      rgt := strg.find_next(Sentence, lft, ">")
      strg.substr(@wrd, Sentence, lft, rgt - lft + 1)
      rule := strg.get_dict(Rules, wrd)
      option := rand(strg.size(rule))
      strg.replace(@Sentence, lft, rgt - lft + 1, word[rule][option])    
    print.f1(string("%s\r"), strg.uc(Sentence, 0))
    waitcnt(cnt + DELAY * clkfreq + 1_000_000)
  while CONTINUOUS

PUB rand(ceil)

  return Seed ? & $7FFFFFFF // ceil    
  
DAT

  grammar     byte      "<sentence> ::= <subj_phrase> <i_verb_phrase>.|<subj_phrase> <t_verb> <obj_phrase>.|<subj_phrase> <adverb> <t_verb> <obj_phrase>./"
              byte      "<subj_phrase> ::= <s_noun_phrase>|<s_noun_phrase> <prep_phrase>/" 
              byte      "<obj_phrase> ::= <o_noun_phrase>|<o_noun_phrase> <prep_phrase>/" 
              byte      "<s_noun_phrase> ::= <name>|<article> <s_noun>|<adj_phrase> <s_noun>/"
              byte      "<o_noun_phrase> ::= <name>|<article> <o_noun>|<adj_phrase> <o_noun>/"
              byte      "<prep_phrase> ::= <preposition> <place>/"
              byte      "<adj_phrase> ::= <article> <s_noun>'s|<name>'s|<article> <s_noun>'s <adjective>|<article> <adjective>|<article> <adjective>, <adjective>|<article> <adjective> but <adjective>/"
              byte      "<i_verb_phrase> ::= <i_verb>|<adverb> <i_verb>|<i_verb> <prep_phrase>|<adverb> <i_verb> <prep_phrase>/"
              byte      "<article> ::= the|his|her/"
              byte      "<adjective> ::= pointy-haired|nervous|lazy|energetic|nice|creepy|smelly|sleazy|purple|overrated|underwhelmed/"
              byte      "<s_noun> ::= boss|brother|sister|spouse|uncle|aunt|slack-jawed yokel|cat|dog|aardvark|duck|frog|programmer|gardener|maid|pool boy|rock star/"
              byte      "<o_noun> ::= <s_noun>|<s_noun>|football|posterior|Propeller|dress|suit|rug|toupee|face|apples|melons/"
              byte      "<place> ::= Parallax|Rocklin|France|the Panama Canal|Washington|Elbonia/"
              byte      "<i_verb> ::= yelled|ran|whistled|cried|swam|quacked|croaked|tooted/"
              byte      "<t_verb> ::= <i_verb> at|kicked|pushed|kissed|bit|patted|poked|praised|chewed on|barfed on|painted/"
              byte      "<adverb> ::= gleefully|smugly|awkwardly|loudly|unwittingly|excitedly|quickly|slowly/"
              byte      "<name> ::= Dilbert|George Bush|Hillary Clinton|Dame Edna|Queen Elizabeth|Britney Spears/"
              byte      "<preposition> ::= in|from", 0
                   